function johansen_output=new_joh_proc(LevelData,XData,L,h,const,dummy);
%[beta, alpha,AR,PI,Omega,epsilon]=joh_proc(LevelData,XData,h,const,dummy)
%ML-procedure for estimating a Vector Error Correction Model
%  Input-Parameters are 
%* LevelData  - the name reminds that the Data have to be provided in levels
%               the routine does the necessary differencing by itself 
%* XData      - exogeneous variables, NOT including the constant term!
%* h          - the number  of cointegrating relations. If h is not provided
%               the routine performs unconstrained estimation
%  const      - define the treatment of the constant term
%		 const=0: unrestricted constant term
%		 const=1: constant term restricted to lie in the CI-space
%  dummy      - define the treatment of dummies or exogeneous Data:
%                dummy=0: all dummies &exogeneous data unrestricted
%                dummy>0: first 'dummy' columns of 'XData' are restricted to 
%                         be in the CI-space.
%  Output:
%* beta       - the cointegrating vectors in columns
%* alpha      - the adjustment coeffcients.
%               In the case unconstrained estimation (i.e. h not provided), 
%               the routine gives alpha=PI, beta=Id(N)
%* AR         - the short-run coefficient-matrices
%* PI         - the LR-matrix
%* Omega      - the variance-covariance
%* epsilon    - the residuals of the VECM
%* R0, R1     - the residuals of the auxiliary regressions
%
%
%copyright Mathias Hoffmann, June 1997.
%Updated to return structures (instead of using globals variables): Sep 2011
%------------------------------------------------------------------------------



[T,N]=size(LevelData);

if (nargin<5);
const=0;%constant unrestricted by default
end;
if nargin<6;
dummy=0;%dummies unrestricted by default.
end;

%--------------------------------------------------------------
%Set everything up.....
L=L-1;

DiffData=LevelData(2:T,:)-LevelData(1:T-1,:);
[Td,N]=size(DiffData);
[t,n]=size(XData);
if L>0;
    dy=DiffData;
    X=lag0(DiffData,(1:L));
    dy=dy(L+1:end,:); X=X(L+1:end,:);
else
X=[];
dy=DiffData;
end;
%----------------------------------------------------------------
%set up constant terms.................
if const==0;
constant=ones(length(dy),1);
CI_constant=[];
else
constant=[];
CI_constant=ones(length(dy),1);
end;
%----------------------------------------------------------------
%set up restricted dummies or exogenous variables in the CI-space
if dummy==0;
RXData=[];
else
RXData=XData(:,1:dummy);
RXData=RXData(L+2:T,:);
[t,n]=size(XData);
XData=XData(:,dummy+1:n);
[t,n]=size(XData);
end;
%----------------------------------------------------------------
if n>1;
X=[constant,XData(L+1:end,:),X];%length(XData)-length(X)
else
X=[constant,X];
end;

y=LevelData(L+1:T-1,:);
Stage2Regr=[y,CI_constant,RXData];
%--------------------------------------------------------------
%Auxiliary Regressions
[A,R0]=OLS(dy,X);
[B,R1]=OLS(Stage2Regr,X);
%--------------------------------------------------------------
%Calculate Canonical Correlations...

S00=1/T.*R0'*R0;
S11=1/T.*R1'*R1;
S10=1/T.*R1'*R0;
S01=S10';

P=S10*inv(S00)*S01;
%--------------------------------------------------------------
if nargin>2;
[beta,EigVal]=geigvec(P,S11,h);
else
beta=inv(S11);
end;

%----------------------------------------------------
%now calculate coefficient estimates
%1.)Long-Run parameters
alpha=S01*beta*inv(beta'*S11*beta);
if nargin>2;
PI=alpha*beta';
else
PI=alpha;
end;

%2.)Short-Run coefficents conditional on alpha*beta'
B=B';
A=A';
A
B
PI
AR=A-PI*B


%pause;
if n>0;
D=AR(:,1:n+1-const);%includes constant!
end;


if L>0;
AR=AR(:,2+n:length(AR));
else
AR=zeros(N,N);
end;



epsilon=R0-R1*PI';
Omega=(1/T).*epsilon'*epsilon;
%or alternatively: 
%Omega=S00-S01*beta*inv(beta'*S11*beta)*beta'*S10;



johansen_output.beta=beta;
johansen_output.alpha=alpha;
johansen_output.AR=AR;
johansen_output.PI=PI;
johansen_output.Omega=Omega;
johansen_output.epsilon=epsilon;
johansen_output.R0=R0;
johansen_output.R1=R1;
if exist('D')
johansen_output.D=D;
end;
end



